@@ -58,10 +58,10 @@ per_particle_vars = (
         (xo.Int64, 'at_turn'),
         (xo.Int64, 'state'),
         (xo.Int64, 'parent_particle_id'),
-        (xo.UInt32, '__rng_s1'),
-        (xo.UInt32, '__rng_s2'),
-        (xo.UInt32, '__rng_s3'),
-        (xo.UInt32, '__rng_s4')
+        (xo.UInt32, '_rng_s1'),
+        (xo.UInt32, '_rng_s2'),
+        (xo.UInt32, '_rng_s3'),
+        (xo.UInt32, '_rng_s4')
     )
     )
 
@@ -149,6 +149,8 @@ class Particles(xo.dress(ParticlesData, rename={
             if any([nn in kwargs.keys() for tt, nn in per_particle_vars]):
                 # Needed to generate consistent longitudinal variables
                 pyparticles = Pyparticles(**kwargs)
+                if 'mass_ratio' in kwargs.keys():
+                    del(kwargs['mass_ratio']) # info transferred in pyparticles.chi
 
                 part_dict = _pyparticles_to_xpart_dict(pyparticles)
                 if ('_capacity' in kwargs.keys() and
@@ -178,7 +180,8 @@ class Particles(xo.dress(ParticlesData, rename={
                     for tt, kk in list(scalar_vars):
                         setattr(self, kk, part_dict[kk])
                     for tt, kk in list(per_particle_vars):
-                        if kk.startswith('__'):
+                        if kk.startswith('_rng'):
+                            getattr(self, kk)[:] = 0
                             continue
                         vv = getattr(self, kk)
                         vals =  context.nparray_to_context_array(part_dict[kk])
@@ -251,7 +254,7 @@ class Particles(xo.dress(ParticlesData, rename={
         if remove_unused_space:
             p_for_dict = p_for_dict.remove_unused_space()
 
-        dct = super(p_for_dict.__class__, p_for_dict).to_dict()
+        dct = Particles.__base__.to_dict(p_for_dict)
         dct['delta'] = p_for_dict.delta
         dct['psigma'] = p_for_dict.psigma
         dct['rvv'] = p_for_dict.rvv
@@ -260,7 +263,7 @@ class Particles(xo.dress(ParticlesData, rename={
         if remove_underscored:
             for kk in list(dct.keys()):
                 if kk.startswith('_'):
-                    if keep_rng_state and kk.startswith('__rng'):
+                    if keep_rng_state and kk.startswith('_rng'):
                         continue
                     del(dct[kk])
 
@@ -409,6 +412,16 @@ class Particles(xo.dress(ParticlesData, rename={
     def _bypass_linked_vars(self):
         return BypassLinked(self)
 
+    def _has_valid_rng_state(self):
+        # I check only the first particle
+        if (self._xobject._rng_s1[0] == 0
+            and self._xobject._rng_s2[0] == 0
+            and self._xobject._rng_s3[0] == 0
+            and self._xobject._rng_s4[0] == 0):
+            return False
+        else:
+            return True
+
     def _init_random_number_generator(self, seeds=None):
 
          self.compile_custom_kernels(only_if_needed=True)
@@ -417,7 +430,7 @@ class Particles(xo.dress(ParticlesData, rename={
             seeds = np.random.randint(low=1, high=4e9,
                         size=self._capacity, dtype=np.uint32)
          else:
-            assert len(seeds) == particles._capacity
+            assert len(seeds) == self._capacity
             if not hasattr(seeds, 'dtype') or seeds.dtype != np.uint32:
                 seeds = np.array(seeds, dtype=np.uint32)
 
@@ -600,7 +613,7 @@ class Particles(xo.dress(ParticlesData, rename={
         for tt, kk in list(scalar_vars):
             setattr(self, kk, part_dict[kk])
         for tt, kk in list(per_particle_vars):
-            if kk.startswith('__') and kk not in part_dict.keys():
+            if kk.startswith('_rng') and kk not in part_dict.keys():
                 continue
             getattr(self, kk)[index] = part_dict[kk][0]
 
@@ -780,29 +793,24 @@ double LocalParticle_get_energy0(LocalParticle* part){
 /*gpufun*/
 void LocalParticle_add_to_energy(LocalParticle* part, double delta_energy, int pz_only ){
 
+    double const psigma = LocalParticle_get_psigma(part);
     double const beta0 = LocalParticle_get_beta0(part);
-    double const delta_beta0 = LocalParticle_get_delta(part) * beta0;
+    double const p0c = LocalParticle_get_p0c(part);
 
-    double const ptau_beta0 =
-        delta_energy / LocalParticle_get_energy0(part) +
-        sqrt( delta_beta0 * delta_beta0 + 2.0 * delta_beta0 * beta0
-                + 1. ) - 1.;
+    double ptau = psigma * beta0;
 
-    double const ptau   = ptau_beta0 / beta0;
-    double const psigma = ptau / beta0;
-    double const delta = sqrt( ptau * ptau + 2. * psigma + 1 ) - 1;
+    ptau += delta_energy/p0c;
 
-    double const one_plus_delta = delta + 1.;
-    double const rvv = one_plus_delta / ( 1. + ptau_beta0 );
+    double const irpp = sqrt(ptau*ptau + 2*ptau/beta0 +1);
 
-    LocalParticle_set_delta(part, delta );
-    LocalParticle_set_psigma(part, psigma );
+    double const new_rpp = 1./irpp;
+    LocalParticle_set_delta(part, irpp - 1.);
+
+    double const new_rvv = irpp/(1 + beta0*ptau);
     LocalParticle_scale_zeta(part,
-        rvv / LocalParticle_get_rvv(part));
-
-    LocalParticle_set_rvv(part, rvv );
-
-    double const new_rpp = 1. / one_plus_delta;
+        new_rvv / LocalParticle_get_rvv(part));
+    LocalParticle_set_rvv(part, new_rvv);
+    LocalParticle_set_psigma(part, ptau/beta0);
 
     if (!pz_only) {
         double const old_rpp = LocalParticle_get_rpp(part);
@@ -810,7 +818,6 @@ void LocalParticle_add_to_energy(LocalParticle* part, double delta_energy, int p
         LocalParticle_scale_px(part, f);
         LocalParticle_scale_py(part, f);
     }
-
     LocalParticle_set_rpp(part, new_rpp );
 }
 
@@ -879,7 +886,7 @@ def _pyparticles_to_xpart_dict(pyparticles):
         dct['weight'] = 1.
 
     for tt, kk in scalar_vars + per_particle_vars:
-        if kk.startswith('__'):
+        if kk.startswith('_rng'):
             continue
         # Use properties
         dct[kk] = getattr(pyparticles, kk)
@@ -900,7 +907,7 @@ def _pyparticles_to_xpart_dict(pyparticles):
         out[kk] = val[0]
 
     for tt, kk in per_particle_vars:
-        if kk.startswith('__'):
+        if kk.startswith('_rng'):
             continue
 
         val_py = dct[kk]
